*****************************************************************************
*	Smooting time series and computing successive time derivatives
*
*                              Original code written by Loïs Le Sceller, 1996
*----------------------------------------------------------------------------
*   npoint    : nombre de points (2^n)
*   nphas     : dimension of the reconstructed phase space
*   nmax      : number of points used for the estimation of the polynomial
*   vois      : Smoothing window
*   h         : time step of the time series
*   dxdt(1)   : time series
*   dxdt(1+i) : i-th time derivative
*****************************************************************************
      implicit none
      integer i,k,nwin,nphas,popo,nmax,npoint,m
      parameter(nmax=20,nphas=3,npoint=16384)
      double precision h,xim(nmax),t0,dxdt(10),y,t,vois,tab(npoint)

      h=0.050                          	! time step of sampling 
      vois=10				! smoothing parameter
*---------------------------------------------------------------------------
      open(12,file='Tcopp.dat')       ! time series file
      open(14,file='attcop.dat')      	! derivatives file
      do i=1,npoint
        read(12,*) tab(i)		! case of a one-column file
      enddo
      close(12)
      call smooft(tab,npoint,vois)	! Smoothing the time series
      popo=12
      nwin=nphas+popo
      t0=0.0
      m=0
      do i=1,nwin
        do k=1,nwin
          xim(k)=xim(k+1)
        enddo
        m=m+1
        y=tab(m)
        xim(nwin)=y
      enddo
      do i=1,npoint-nwin
        do k=1,nwin
          xim(k)=xim(k+1)
        enddo
        m=m+1
        xim(nwin)=tab(m)
        call deriv(xim,t0,h,nwin,dxdt,nphas+1) 		! Computing time derivatives
        write(14,*) dxdt(1),dxdt(2),dxdt(3)
      enddo
      close(14)
      call system ("xmgrace attcop.dat")		! Call the graphic software
      end
*****************************************************************************
           subroutine deriv(x,t0,dt,n,dxdt,imax)
*             calcul de derivees avec extrapolation polynomiale
*   n sera forcement impair, les derivees seront calculees en x((n+1)/2)
*****************************************************************************
      implicit none
      integer n,nmax,nmil,i,j,k,fac,imax
      parameter(nmax=20)
      double precision x(nmax),t0,dt,dxdt(nmax)
      double precision t(nmax),coeff(nmax),tderiv(0:nmax)

      nmil=(n+1)/2
      t(1)=t0
      do i=2,n
        t(i)=t(i-1)+dt
      enddo
      tderiv(0)=1.d00
      do i=1,n-1
        tderiv(i)=tderiv(i-1)*t(nmil)
      enddo
      call interpoly(x,t,n,coeff)
      dxdt(1)=x(nmil)
      do j=1,imax-1
        dxdt(j+1)=0.d00
        do i=j+1,n
          fac=1
          do k=i-j,i-1,1
            fac=fac*k
          enddo
          dxdt(j+1)=dxdt(j+1)+fac*coeff(i)*tderiv(i-j-1)
        enddo
      enddo
      return
      end
*****************************************************************************
                  subroutine svbksb(u,w,v,m,n,mp,np,b,x)
*                   resolution avec la decomposition en SVD
*****************************************************************************
      implicit real*8 (a-h,o-z)
      parameter (nmax=20)
      dimension u(mp,np),w(np),v(np,np),b(mp),x(np),tmp(nmax)

      do 12 j=1,n
            s=0.d0
            if(w(j).ne.0.d0)then
                    do 11 i=1,m
                          s=s+u(i,j)*b(i)
11                  continue
                    s=s/w(j)
            endif
            tmp(j)=s
12    continue

      do 14 j=1,n
            s=0.d0
            do 13 jj=1,n
                  s=s+v(j,jj)*tmp(jj)
13          continue
            x(j)=s
14    continue
      return
      end
*****************************************************************************
         subroutine svdcmp(a,m,n,mp,np,w,v)
*                            decomposition en SVD
*****************************************************************************
      implicit real*8 (a-h,o-z)
      parameter (nmax=20)
      dimension a(mp,np),w(np),v(np,np),rv1(nmax)

      g=0.0d0
      scale=0.0d0
      anorm=0.0d0
      do 25 i=1,n
        l=i+1
        rv1(i)=scale*g
        g=0.0d0
        s=0.0d0
        scale=0.0d0
        if (i.le.m) then
          do 11 k=i,m
            scale=scale+dabs(a(k,i))
11        continue
          if (scale.ne.0.0d0) then
            do 12 k=i,m
              a(k,i)=a(k,i)/scale
              s=s+a(k,i)*a(k,i)
12          continue
            f=a(i,i)
            g=-dsign(dsqrt(s),f)
            h=f*g-s
            a(i,i)=f-g
            if (i.ne.n) then
              do 15 j=l,n
                s=0.0d0
                do 13 k=i,m
                  s=s+a(k,i)*a(k,j)
13              continue
                f=s/h
                do 14 k=i,m
                  a(k,j)=a(k,j)+f*a(k,i)
14              continue
15            continue
            endif
            do 16 k= i,m
              a(k,i)=scale*a(k,i)
16          continue
          endif
        endif
        w(i)=scale *g
        g=0.0d0
        s=0.0d0
        scale=0.0d0
        if ((i.le.m).and.(i.ne.n)) then
          do 17 k=l,n
            scale=scale+dabs(a(i,k))
17        continue
          if (scale.ne.0.0d0) then
            do 18 k=l,n
              a(i,k)=a(i,k)/scale
              s=s+a(i,k)*a(i,k)
18          continue
            f=a(i,l)
            g=-dsign(dsqrt(s),f)
            h=f*g-s
            a(i,l)=f-g
            do 19 k=l,n
              rv1(k)=a(i,k)/h
19          continue
            if (i.ne.m) then
              do 23 j=l,m
                s=0.0d0
                do 21 k=l,n
                  s=s+a(j,k)*a(i,k)
21              continue
                do 22 k=l,n
                  a(j,k)=a(j,k)+s*rv1(k)
22              continue
23            continue
            endif
            do 24 k=l,n
              a(i,k)=scale*a(i,k)
24          continue
          endif
        endif
        anorm=dmax1(anorm,(dabs(w(i))+dabs(rv1(i))))
25    continue
      do 32 i=n,1,-1
        if (i.lt.n) then
          if (g.ne.0.0d0) then
            do 26 j=l,n
              v(j,i)=(a(i,j)/a(i,l))/g
26          continue
            do 29 j=l,n
              s=0.0d0
              do 27 k=l,n
                s=s+a(i,k)*v(k,j)
27            continue
              do 28 k=l,n
                v(k,j)=v(k,j)+s*v(k,i)
28            continue
29          continue
          endif
          do 31 j=l,n
            v(i,j)=0.0d0
            v(j,i)=0.0d0
31        continue
        endif
        v(i,i)=1.0d0
        g=rv1(i)
        l=i
32    continue
      do 39 i=n,1,-1
        l=i+1
        g=w(i)
        if (i.lt.n) then
          do 33 j=l,n
            a(i,j)=0.0d0
33        continue
        endif
        if (g.ne.0.0d0) then
          g=1.0d0/g
          if (i.ne.n) then
            do 36 j=l,n
              s=0.0d0
              do 34 k=l,m
                s=s+a(k,i)*a(k,j)
34            continue
              f=(s/a(i,i))*g
              do 35 k=i,m
                a(k,j)=a(k,j)+f*a(k,i)
35            continue
36          continue
          endif
          do 37 j=i,m
            a(j,i)=a(j,i)*g
37        continue
        else
          do 38 j= i,m
            a(j,i)=0.0d0
38        continue
        endif
        a(i,i)=a(i,i)+1.0d0
39    continue
      do 49 k=n,1,-1
        do 48 its=1,30
          do 41 l=k,1,-1
            nm=l-1
            if ((dabs(rv1(l))+anorm).eq.anorm)  go to 2
            if ((dabs(w(nm))+anorm).eq.anorm)  go to 1
41        continue
1         c=0.0d0
          s=1.0d0
          do 43 i=l,k
            f=s*rv1(i)
            if ((dabs(f)+anorm).ne.anorm) then
              g=w(i)
              h=dsqrt(f*f+g*g)
              w(i)=h
              h=1.0d0/h
              c= (g*h)
              s=-(f*h)
              do 42 j=1,m
                y=a(j,nm)
                z=a(j,i)
                a(j,nm)=(y*c)+(z*s)
                a(j,i)=-(y*s)+(z*c)
42            continue
            endif
43        continue
2         z=w(k)
          if (l.eq.k) then
            if (z.lt.0.0d0) then
              w(k)=-z
              do 44 j=1,n
                v(j,k)=-v(j,k)
44            continue
            endif
            go to 3
          endif
          if (its.eq.30) pause 'no convergence in 30 iterations'
          x=w(l)
          nm=k-1
          y=w(nm)
          g=rv1(nm)
          h=rv1(k)
          f=((y-z)*(y+z)+(g-h)*(g+h))/(2.0d0*h*y)
          g=sqrt(f*f+1.0d0)
          f=((x-z)*(x+z)+h*((y/(f+dsign(g,f)))-h))/x
          c=1.0d0
          s=1.0d0
          do 47 j=l,nm
            i=j+1
            g=rv1(i)
            y=w(i)
            h=s*g
            g=c*g
            z=dsqrt(f*f+h*h)
            rv1(j)=z
            c=f/z
            s=h/z
            f= (x*c)+(g*s)
            g=-(x*s)+(g*c)
            h=y*s
            y=y*c
            do 45 nm=1,n
              x=v(nm,j)
              z=v(nm,i)
              v(nm,j)= (x*c)+(z*s)
              v(nm,i)=-(x*s)+(z*c)
45          continue
            z=dsqrt(f*f+h*h)
            w(j)=z
            if (z.ne.0.0d0) then
              z=1.0d0/z
              c=f*z
              s=h*z
            endif
            f= (c*g)+(s*y)
            x=-(s*g)+(c*y)
            do 46 nm=1,m
              y=a(nm,j)
              z=a(nm,i)
              a(nm,j)= (y*c)+(z*s)
              a(nm,i)=-(y*s)+(z*c)
46          continue
47        continue
          rv1(l)=0.0d0
          rv1(k)=f
          w(k)=x
48      continue
3       continue
49    continue
      return
      end
*****************************************************************************
           subroutine interpoly(x,t,n,coeff)
*                      interpolation polynomiale
*****************************************************************************
      implicit none
      integer n,nmax,i,j
      parameter(nmax=20)
      double precision x(nmax),t(nmax),coeff(nmax),
     smat(nmax,nmax),w(nmax),v(nmax,nmax),wmax,wmin

      do i=1,n
         mat(i,1)=1.d00
         do j=2,n
            mat(i,j)=mat(i,j-1)*t(i)
         enddo
      enddo
      call svdcmp(mat,n,n,nmax,nmax,w,v)
      wmax=0.d00
      do i=1,n
         if (dabs(w(i)).gt.wmax) then
                      wmax=dabs(w(i))
         endif
      enddo
      wmin=wmax*1.d-12
      do i=1,n
         if (dabs(w(i)).lt.wmin) then
                           w(i)=0.d00
         endif
      enddo
      call svbksb(mat,w,v,n,n,nmax,nmax,x,coeff)
      return
      end
c****************************************************************************
                      subroutine smooft(y,n,pts)
c****************************************************************************
      double precision y1,yn,y(n),const
      real*8 pts
      integer n,j,mmax
c
      mmax=n
      const=(pts/dble(mmax))**2
      y1=y(1)
      yn=y(n)
      rn1=1.d00/dble(n-1)
      do j=1,n
        y(j)=y(j)-rn1*(y1*dble(n-j)+yn*dble(j-1))
      enddo
      if (n+1.le.mmax) then
          do j=n+1,mmax
            y(j)=0.d00
          enddo
      endif
      m02=mmax/2
      call realft(y,m02,1)
      y(1)=y(1)/dble(m02)
      fac=1.d00
      do j=1,m02-1
        k=2*j+1
        if (fac.ne.0.d00) then
            fac=dmax1(0.d00,(1.d00-const*j**2)/dble(m02))
            y(k)=fac*y(k)
            y(k+1)=fac*y(k+1)
        else
            y(k)=0.d00
            y(k+1)=0.d00
        endif
      enddo
      fac=dmax1(0.d00,(1.d00-0.25d00*dble(pts**2))/dble(m02))
      y(2)=fac*y(2)
      call realft(y,m02,-1)
      do j=1,n
        y(j)=rn1*(y1*dble(n-j)+yn*dble(j-1))+y(j)
      enddo
      return
      end
c******************************************************************
      SUBROUTINE REALFT(DATA,N,ISIGN)
c******************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 WR,WI,WPR,WPI,WTEMP,THETA
      DIMENSION DATA(*)
      THETA=6.28318530717959D0/2.0D0/DBLE(N)
      C1=0.5D0
      IF (ISIGN.EQ.1) THEN
        C2=-0.5D0
        CALL FOUR1(DATA,N,+1)
      ELSE
        C2=0.5D0
        THETA=-THETA
      ENDIF
      WPR=-2.0D0*DSIN(0.5D0*THETA)**2
      WPI=DSIN(THETA)
      WR=1.0D0+WPR
      WI=WPI
      N2P3=2*N+3
      DO 11 I=2,N/2+1
        I1=2*I-1
        I2=I1+1
        I3=N2P3-I2
        I4=I3+1
        H1R=C1*(DATA(I1)+DATA(I3))
        H1I=C1*(DATA(I2)-DATA(I4))
        H2R=-C2*(DATA(I2)+DATA(I4))
        H2I=C2*(DATA(I1)-DATA(I3))
        DATA(I1)=H1R+WR*H2R-WI*H2I
        DATA(I2)=H1I+WR*H2I+WI*H2R
        DATA(I3)=H1R-WR*H2R+WI*H2I
        DATA(I4)=-H1I+WR*H2I+WI*H2R
        WTEMP=WR
        WR=WR*WPR-WI*WPI+WR
        WI=WI*WPR+WTEMP*WPI+WI
11    CONTINUE
      IF (ISIGN.EQ.1) THEN
        H1R=DATA(1)
        DATA(1)=H1R+DATA(2)
        DATA(2)=H1R-DATA(2)
      ELSE
        H1R=DATA(1)
        DATA(1)=C1*(H1R+DATA(2))
        DATA(2)=C1*(H1R-DATA(2))
        CALL FOUR1(DATA,N,-1)
      ENDIF
      RETURN
      END
c******************************************************************
      SUBROUTINE FOUR1(DATA,NN,ISIGN)
c******************************************************************
      IMPLICIT REAL*8 (A-H,O-Z)
      REAL*8 WR,WI,WPR,WPI,WTEMP,THETA
      DIMENSION DATA(*)
      N=2*NN
      J=1
      DO 11 I=1,N,2
        IF(J.GT.I)THEN
          TEMPR=DATA(J)
          TEMPI=DATA(J+1)
          DATA(J)=DATA(I)
          DATA(J+1)=DATA(I+1)
          DATA(I)=TEMPR
          DATA(I+1)=TEMPI
        ENDIF
        M=N/2
1       IF ((M.GE.2).AND.(J.GT.M)) THEN
          J=J-M
          M=M/2
        GO TO 1
        ENDIF
        J=J+M
11    CONTINUE
      MMAX=2
2     IF (N.GT.MMAX) THEN
        ISTEP=2*MMAX
        THETA=6.28318530717959D0/(ISIGN*MMAX)
        WPR=-2.D0*DSIN(0.5D0*THETA)**2
        WPI=DSIN(THETA)
        WR=1.D0
        WI=0.D0
        DO 13 M=1,MMAX,2
          DO 12 I=M,N,ISTEP
            J=I+MMAX
            TEMPR=WR*DATA(J)-WI*DATA(J+1)
            TEMPI=WR*DATA(J+1)+WI*DATA(J)
            DATA(J)=DATA(I)-TEMPR
            DATA(J+1)=DATA(I+1)-TEMPI
            DATA(I)=DATA(I)+TEMPR
            DATA(I+1)=DATA(I+1)+TEMPI
12        CONTINUE
          WTEMP=WR
          WR=WR*WPR-WI*WPI+WR
          WI=WI*WPR+WTEMP*WPI+WI
13      CONTINUE
        MMAX=ISTEP
      GO TO 2
      ENDIF
      RETURN
      END
